package gus05.manager.gus.kassia.layer3.t05.initfileinitializer;


import java.io.File;
import java.util.HashSet;

import javax.swing.JOptionPane;

import gus05.framework.features.Register;
import gus05.manager.gus.kassia.layer3.KS3_data;

public class T05 extends KS3_data implements Register {

	
	
	public static void initFileInitializer() throws Exception
	{putPool(POOL_REGISTER_FILEINITIALIZER,new T05());}
	
	
	
	
	public void register(String key, Object obj) throws Exception
	{initialize(key,(File)obj);}
	
	
	
	
	private void initialize(String key, File f) throws Exception
	{
		if(f==null)
        	throw new Exception("Attempt to initialize null file object");
        
        if(f.exists())
        {
        	if(f.isFile() && key.equals("dir"))
        		throw new Exception("Attempt to initialize existing file as dir");
        	if(f.isDirectory() && key.equals("file"))
        		throw new Exception("Attempt to initialize existing dir as file");
        	return;
        }
        	
        if(key.equals("dir")) {initializeDir(f); return;}
        if(key.equals("file")) {initializeFile(f); return;}
        throw new Exception("Unknown key: ["+key+"]");
	}
	
	
	
	private void initializeDir(File dir)
    {
        if(!acceptCreation(dir,"directory")) return;
        
        try{mkdirs(dir);}
        catch(Exception e)
        {err(this,"initializeDir(File)",e);}
    }
    
    
    
    private void initializeFile(File file)
    {
        if(!acceptCreation(file,"file")) return;
        
        try
        {
        	File parent = file.getParentFile();
        	if(parent!=null) mkdirs(parent);
        	file.createNewFile();
        }
        catch(Exception e)
        {err(this,"initializeFile(File)",e);}
    }
    
    
    
    private void mkdirs(File dir) throws Exception
    {
    	if(dir.exists()) return;
    	if(!dir.mkdirs()) throw new Exception("Dir creation failed: ["+dir+"]");
    }
    
    
    
    
    
    
    
    private boolean acceptCreation(File f, String type)
    {
    	String level = getProp(PROPKEY_PATHCREATIONLEVEL,null);
        if(level!=null && level.equals(PROPVALUE_PATHCREATIONLEVEL_NEVERASK)) return true;
        
        
    	boolean insideRootDir = insideRootDir(f);
        String where = insideRootDir?"inside the root directory":"somewhere on your computer";
        
        if(level==null)
        	return askForUserConfirmation(f,type,where);
        
        if(level.equals(PROPVALUE_PATHCREATIONLEVEL_ALWAYSASK))
        	return askForUserConfirmation(f,type,where);
        
        if(level.equals(PROPVALUE_PATHCREATIONLEVEL_ASKOUTSIDEROOT))
        	return insideRootDir?true:askForUserConfirmation(f,type,where);
        
        return true;
    }
    
    
    
    
    private HashSet alreadyAsked = new HashSet();
    
    private boolean askForUserConfirmation(File f, String type, String where)
    {
    	String path = f.getAbsolutePath();
    	if(alreadyAsked.contains(path)) return false;
    	alreadyAsked.add(path);
    	
        String message = "Application attempts to create the "+type+" :\n["+path+"]\n"+where+". Do you agree?";
        String title = type+" creation warning";
        int r = JOptionPane.showConfirmDialog(null,message,title,JOptionPane.YES_NO_OPTION);
        return r==JOptionPane.YES_OPTION;
    }
    
    
    
    private boolean insideRootDir(File f)
    {
    	File rootDir = file(FILEID_ROOTDIR);
    	if(rootDir==null) return false;
        String rootPath = rootDir.getAbsolutePath();
        return f.getAbsolutePath().startsWith(rootPath);
    }
}
