 package gus05.manager.gus.kassia.layer3.t06.initfileprovider;

import java.io.File;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import gus05.framework.resources.FileProvider;
import gus05.manager.gus.kassia.layer3.KS3_data;


public class T06 extends KS3_data implements FileProvider {

	public static void initFileProvider() throws Exception
	{
        putPoolMap(POOL_MAP_FILES);
        putPool(POOL_FILEPROVIDER,new T06());
	}
	
	
    private T06()
    {
        loadFromSystemProperties();
        loadFromSystemEnv();
    }
    
    
    
    private void loadFromSystemProperties()
    {
        Properties sysProp = System.getProperties();
        Iterator it = sysProp.keySet().iterator();
        
        while(it.hasNext())
        {
            String key = (String)it.next();
            String value = System.getProperty(key);
            loadFromLine("sys."+key,value);
        }
    }
    
    
    private void loadFromSystemEnv()
    {
        Map envProp = System.getenv();
        Iterator it = envProp.keySet().iterator();
        
        while(it.hasNext())
        {
            String key = (String)it.next();
            String value = System.getenv(key);
            loadFromLine("env."+key,value);
        }
    }
    
    
    private void loadFromLine(String key, String value)
    {
        String[] paths = value.split(File.pathSeparator);
        
        for(int i=0;i<paths.length;i++)
        {
            String path = paths[i];
            if(path.length()>0)
            {
                File f = new File(path);
                if(f.exists())
                {
                    if(paths.length==1) putFile("path."+key,f);
                    else putFile("path."+key+"."+i,f);
                }
            }
        }
    }
    
    
    
    
    
    private void putFile(String key, File f)
    {map_files().put(key,f);}
    
    
    
    public File getFile(String key)
    {
    	try{return searchFile(key,0);}
    	catch(Exception e)
    	{
			String message = "Failed to provide file with pathID=["+key+"]";
			err(this,"getFile(String)",new Exception(message,e));
		}
    	return null;
    }
    
    
    
    
    
    
    
    
    
    
    

    
    
    
    
    private static final Pattern pattern_tag = Pattern.compile("<([^>]+)>");
    private static final Pattern pattern_move = Pattern.compile("\\[(-?[0-9]+)\\]");
    
    private static final String GUSKLING_FILENAME = "guslink.txt";
    private static final int LOOKUP_LIMIT = 200;
    
    
    
    
    private File searchFile(String info, int c) throws Exception
    {
    	if(c>LOOKUP_LIMIT)
    		throw new Exception("Infinite loop detected: stopping at value "+info);
    	
    	if(map_files().containsKey(info))
    		return (File) map_files().get(info);

    	
    	
    	
    	if(info.startsWith("path."))
    	{
    		if(!hasProp(info)) return null;
    		String newInfo = getProp(info);
    		if(newInfo.equals("null")) return null;
    		
    		File f = searchFile(newInfo,++c);
    		if(f==null) return null;
    		putFile(info,f);
    		return f;
    	}
    	
    	if(info.startsWith("["))
    	{
    		Matcher m = pattern_move.matcher(info);
            if(!m.find()) throw new Exception("Wrong syntax for fileID rule: ["+info+"]");
            if(m.start()>0) throw new Exception("Wrong syntax for fileID rule: ["+info+"]");
            
            int move = Integer.parseInt(m.group(1));
            String tail = info.substring(m.end());
            
            File f_tail = searchFile(tail,++c);
            if(f_tail==null) return null;
            File file = moveFile(f_tail,move);
            return redirect(file,c);
    	}

    	if(info.startsWith("<"))
    	{
    		Matcher m = pattern_tag.matcher(info);
            if(!m.find()) throw new Exception("Wrong syntax for fileID rule: ["+info+"]");
            if(m.start()>0) throw new Exception("Wrong syntax for fileID rule: ["+info+"]");
            
            String tag = m.group(1);
            String tail = info.substring(m.end());
            
            File f_tag = searchFile(tag,++c);
            if(f_tag==null) return null;
            File file = newFileOS(f_tag,tail);
            return redirect(file,c);
    	}
    	
    	File file = newFileOS(info);
        return redirect(file,c);
    }
    
    
    
    
    
    
    
    

    
    
    private File moveFile(File file, int move)
    {
    	if(!file.isAbsolute())
    		file = file.getAbsoluteFile();
    	
    	int delta = move<0?move*-1:deep(file)-move;
    	
    	for(int i=0;i<delta;i++)
    		if(file.getParentFile()!=null)
    	file = file.getParentFile();
    	
    	return file;
    }
    
    
    
    
    private int deep(File file)
    {
    	int d = 0;
    	while(file.getParentFile()!=null)
    	{file = file.getParentFile();d++;}
    	return d;
    }
    
    
    
    
    
    
    private File redirect(File f, int c) throws Exception
    {
    	if(f==null || !f.exists() || !f.isDirectory()) return f;
    	
    	File linkFile = new File(f,GUSKLING_FILENAME);
    	if(!linkFile.exists() || !linkFile.isFile()) return f;
    	
    	String link = text(linkFile);
    	return searchFile(link,++c);
    }
}
