package gus05.manager.gus.kassia.layer3.t10.loadicons;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.io.File;
import java.net.URL;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.accessibility.AccessibleContext;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import gus05.framework.resources.IconProvider;
import gus05.manager.gus.kassia.layer3.tools.AbstractResourceLoader;



public class T10 extends AbstractResourceLoader {

	
	
	public static void loadIcons() throws Exception
    {
		putPoolMap(POOL_MAP_ICONS);
		
        putPool(POOL_ICONPROVIDER1,new IconProvider(){
			public Icon getIcon(String key) {return icon1(key);}
		});
        putPool(POOL_ICONPROVIDER,new IconProvider(){
			public Icon getIcon(String key) {return icon(key);}
		});
        
        new T10().load("icon",PROPKEY_SEQUENCE_LOADICON,RESOURCEPATH_ICON);
    }
	
	
	
	

	protected void load_fromInside_(int index, String resourceID) throws Exception
	{
		Set set = insideListing(resourceID);
		if(set==null) return;

		Iterator it = set.iterator();
		while(it.hasNext())
		{
			String key = (String)it.next();
			URL url = insideURL(resourceID+"/"+key);
			String name = getResourceName(key);
			loadIcon(name,url);
		}
	}
    
    
    
    
	protected void load_fromPathID_(int index, String pathID) throws Exception
	{
		File f = file(pathID);
		if(f!=null && f.exists())
		{
			if(f.isDirectory())	loadIcon_fromDir(f);
			else if(f.isFile())	loadIcon_fromFile(f);
		}
	}
    
    
    
    
    
    
    
    
    
    private void loadIcon_fromDir(File dir) throws Exception
    {
    	File[] gifs = dir.listFiles(gifFileFilter);
    	for(int i=0;i<gifs.length;i++) loadIcon_fromImageFile(gifs[i]);
    	
    	File[] pngs = dir.listFiles(pngFileFilter);
    	for(int i=0;i<pngs.length;i++) loadIcon_fromImageFile(pngs[i]);
    	
    	File[] icos = dir.listFiles(icoFileFilter);
    	for(int i=0;i<icos.length;i++) loadIcon_fromIcoFile(icos[i]);
    	
    	File[] curs = dir.listFiles(curFileFilter);
    	for(int i=0;i<curs.length;i++) loadIcon_fromIcoFile(curs[i]);
    }
    
    
    
    
    private void loadIcon_fromFile(File file) throws Exception
    {
    	if(gifFileFilter.accept(file))			loadIcon_fromImageFile(file);
		else if(pngFileFilter.accept(file)) 	loadIcon_fromImageFile(file);
		else if(icoFileFilter.accept(file)) 	loadIcon_fromIcoFile(file);
		else if(curFileFilter.accept(file)) 	loadIcon_fromIcoFile(file);
		else if(propFileFilter.accept(file)) 	loadIcon_fromProp(file);
    }
    
    
    
    
    
    private void loadIcon_fromImageFile(File f) throws Exception
    {
    	String name = getResourceName(f);
    	loadIcon(name,f.toURI().toURL());
    }
    
    
    private void loadIcon_fromIcoFile(File f) throws Exception
    {
    	String name = getResourceName(f);
    	loadIcon(name,f);
    }
    
    
    
    
    private void loadIcon_fromProp(File propFile) throws Exception
    {
    	Properties p = prop(propFile);
    	Iterator it = p.keySet().iterator();
    	while(it.hasNext())
    	{
    		String key = (String) it.next();
    		String id = p.getProperty(key);
    		File f = file(id);
    		
    		if(f==null)		{printW(T10.class,"loadIcon_fromOS(File)","FileID not found for OS icon: ["+id+"]");return;}
    		if(!f.exists())	{printW(T10.class,"loadIcon_fromOS(File)","File not found for OS icon: ["+f+"]");return;}
    		
    		loadIcon(key,f);
    	}
    }
    
    

    
    
    private void loadIcon(String key, Object source) throws Exception
    {putIcon(key,buildIcon(key,source));}
    
    
    private void putIcon(String name, Icon icon)
    {map_icons().put(name,icon);}
    
    
    
    
    
    
    
    
    
    private Icon buildIcon(String key, Object source) throws Exception
    {
    	if(source==null) throw new Exception("Source not found for icon name: ["+key+"]");
        if(source instanceof URL) return buildIcon_URL((URL)source);
    	if(source instanceof File) return buildIcon_File((File)source);
    	throw new Exception("Invalid source type: ["+source.getClass().getName()+"] for icon name: ["+key+"]");
    }
    
    private Icon buildIcon_URL(URL url)
    {
    	if(booleanProp(PROPKEY_ICON_LAZYLOAD,false))
    		return new ImageIcon_lazy(url);
    	else return loadIcon_URL(url);
    }
    
    private Icon buildIcon_File(File file)
    {
    	if(booleanProp(PROPKEY_ICON_LAZYLOAD,false))
    		return new Icon_lazy(file);
    	else return loadIcon_File(file);
    }
    
    
    
    
    
    
    
    private class ImageIcon_lazy extends ImageIcon
    {
    	private URL url;
    	private ImageIcon icon;
    	
    	public ImageIcon_lazy(URL url)
    	{this.url = url;}
    	
    	private void init()
    	{if(icon==null) icon = loadIcon_URL(url);}
    	
    	public AccessibleContext getAccessibleContext()
    	{init();return icon.getAccessibleContext();}
         
    	public String getDescription()
    	{init();return icon.getDescription();}
         
    	public int getIconHeight()
    	{init();return icon.getIconHeight();}
         
    	public int getIconWidth()
    	{init();return icon.getIconWidth();}

    	public Image getImage()
    	{init();return icon.getImage();}

    	public int getImageLoadStatus()
    	{init();return icon.getImageLoadStatus();}

    	public ImageObserver getImageObserver()
    	{init();return icon.getImageObserver();}
    	
    	public void paintIcon(Component c, Graphics g, int x, int y)
    	{init();icon.paintIcon(c,g,x,y);}
    	
     	public void setDescription(String description)
     	{init();icon.setDescription(description);}
     	
     	public void setImage(Image image)
     	{init();icon.setImage(image);}
     	
     	public void setImageObserver(ImageObserver observer)
     	{init();icon.setImageObserver(observer);}
     	
     	public String toString()
     	{init();return icon.toString();}
    }
    
    
    
    
    
    
    
    
    
    private class Icon_lazy implements Icon
    {
    	private File file;
    	private Icon icon;
    	
    	public Icon_lazy(File file)
    	{this.file = file;}
    	
    	private void init()
    	{if(icon==null) icon = loadIcon_File(file);}
    	
    	public int getIconHeight()
    	{init();return icon.getIconHeight();}
         
    	public int getIconWidth()
    	{init();return icon.getIconWidth();}

		public void paintIcon(Component c, Graphics g, int x, int y)
		{init();icon.paintIcon(c,g,x,y);}
    }
}
