package gus05.manager.gus.kassia.layer3.t19.initentityclassloader;

import java.io.File;

import gus05.framework.features.Retrieve;
import gus05.framework.features.Transform;
import gus05.manager.gus.kassia.layer3.KS3_data;

public class T19 extends KS3_data implements Retrieve, Transform {

	
	
	public static void initEntityClassLoader() throws Exception
	{
		putPoolMap(POOL_MAP_ENTITYORIGINES);
    	putPool(POOL_RETRIEVE_ENTITYCLASSLOADER,new T19());
    }
	
	
	
	
	public Object retrieve(String key) throws Exception
	{return loadEntityClass(key);}
	
	
	public Object transform(Object obj) throws Exception
	{return loadEntityClass((String)obj);}
	
	
	
	
	private Class loadEntityClass(String name) throws Exception
	{
		try
    	{
			Retrieve classLoader2 = poolret(POOL_RETRIEVE_ENTITYCLASSLOADER2);
			boolean hasSequence = hasProp(PROPKEY_SEQUENCE_LOADENTITYCLASS);
			if(!hasSequence && classLoader2==null)
				throw new Exception("No way found to load entity classes (no sequence, no alternative loader)");
			
			if(hasSequence)
			{
				Class c = loadEntityClass_fromSequence(name);
				if(c!=null) return c;
			}
			if(classLoader2!=null)
				return (Class) classLoader2.retrieve(name);
			return null;
    	}
		catch(Exception e)
    	{
			String message = "Entity class loading failed: ["+name+"]";
            throw new Exception(message,e);
		}
	}
	
	
	
	
	
	
	private Class loadEntityClass_fromSequence(String name) throws Exception
	{
		String seq = getProp(PROPKEY_SEQUENCE_LOADENTITYCLASS);
        if(seq.equals("")) throw new Exception("Load Entity class sequence is empty");
        
        String[] sequence = seq.split(";");
        for(int i=0;i<sequence.length;i++)
        {
            String s = sequence[i];
            if(s.startsWith("path."))
            {
            	Class c = loadEntityClass_fromPath(s,name);
            	if(c!=null) return c;
            }
            else throw new Exception("Invalid Load Entity class sequence: ["+seq+"]");
        }
        return null;
	}
	
	
	
	
	
	
	
	private Class loadEntityClass_fromPath(String path, String name) throws Exception
	{
		File f = file(path);
		if(f==null) throw new Exception("Null value for pathID: ["+path+"]");
		if(!f.exists()) throw new Exception("Path not found: ["+f+"] for pathID: ["+path+"]");

		if(f.isDirectory())
			return loadEntityClass_fromDir(f,name);
		if(jarFileFilter.accept(f))
			return loadEntityClass_fromJar(f,name);
		throw new Exception("Entity class loading not supported for path ["+f+"]");
	}
	
	
	
	
	
	
	
	private Class loadEntityClass_fromDir(File dir, String name) throws Exception
	{
		try
		{
			File jarFile = new File(dir,name+".jar");
			if(jarFile.isFile())
				return loadEntityClass_fromJar(jarFile,name);
			
			String packagePath = ("gus05.entity."+name).replace(".",File.separator);
			File packageDir = new File(dir,packagePath);
			if(packageDir.isDirectory())
				return loadEntityClass_fromBin(dir,name);
			
			File entityDir = new File(dir,name);
			if(entityDir.isDirectory())
			{
				jarFile = searchForJarInside(entityDir);
				if(jarFile!=null && jarFile.isFile())
					return loadEntityClass_fromJar(jarFile,name);
			}
			return null;
		}
		catch(Exception e)
		{
			String message = "Entity class loading ["+name+"] failed from dir ["+dir+"]";
			throw new Exception(message,e);
		}
	}
	
	
	
	
	
	
	private File searchForJarInside(File dir)
	{
		File[] f = dir.listFiles();
		for(int i=0;i<f.length;i++)
		{
			if(jarFileFilter.accept(f[i])) return f[i];
			if(f[i].isDirectory())
			{
				File h = searchForJarInside(f[i]);
				if(h!=null) return h;
			}
		}
		return null;
	}
	
	
	
	
	
	
	
	
	private Class loadEntityClass_fromJar(File jar, String name) throws Exception
	{
		try
		{
			Transform t = entityLoader_jar();
			if(t==null) throw new Exception("No jar entity class loader available inside pool");
			Retrieve ret = (Retrieve) t.transform(jar);
			if(ret==null) throw new Exception("Jar entity class loader returned a null value");
			Class c = (Class) ret.retrieve(name);
			
			if(c!=null) map_entityOrigines().put(name,jar);
			return c;
		}
		catch(Exception e)
		{
			String message = "Entity class loading ["+name+"] failed from jar ["+jar+"]";
			throw new Exception(message,e);
		}
	}
	
	
	
	
	
	
	private Class loadEntityClass_fromBin(File bin, String name) throws Exception
	{
		try
		{
			Transform t = entityLoader_bin();
			if(t==null) throw new Exception("No bin entity class loader available inside pool");
			Retrieve ret = (Retrieve) t.transform(bin);
			if(ret==null) throw new Exception("Bin entity class loader returned a null value");
			Class c = (Class) ret.retrieve(name);
			
			if(c!=null) map_entityOrigines().put(name,bin);
			return c;
		}
		catch(Exception e)
		{
			String message = "Entity class loading ["+name+"] failed from bin ["+bin+"]";
			throw new Exception(message,e);
		}
	}
}
