package gus05.manager.gus.kassia.layer3.t25.initdatabuilder;


import java.io.File;
import java.io.InputStream;
import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.swing.tree.TreeModel;

import gus05.framework.features.Give;
import gus05.framework.features.Provide;
import gus05.framework.features.Retrieve;
import gus05.manager.gus.kassia.layer3.KS3_data;


public class T25 extends KS3_data implements Give {

	
	
	public static void initDataBuilder() throws Exception
	{
        putPoolMap(POOL_MAP_DATASOURCE);
        putPool(POOL_GIVE_DATABUILDER,new T25());
	}
	
	
	
	public void give(Object obj) throws Exception
	{buildData((String)obj);}
	
	
	
	
	
	
	
	private void buildData(String id) throws Exception
    {
        if(!id.contains("#"))
        	throw new Exception("Invalid dataID: ["+id+"]");
        
        String[] n = id.split("#",2);
        String type = n[0];
        String sourceName = n[1];
        
        Object src = findSource(sourceName);
        if(src!=null) registerSource(id,src);

    	Object data = buildData(id,type,src);
    	registerData(id,data);
    }
	
	
	
	
	
	
	
	
	private Object findSource(String sourceName) throws Exception
	{
		Retrieve r = dataSourceFinder();
		if(r==null) throw new Exception("Data source finder not available inside pool");
		return r.retrieve(sourceName);
	}
	
	
	
	
	
	private Object buildData(String id, String type, Object src) throws Exception
	{
		if(type.equals(PROVIDER_DATALIST)) return buildList(id,src);
    	if(type.equals(PROVIDER_DATASET)) return buildSet(id,src);
    	if(type.equals(PROVIDER_DATAMAP)) return buildMap(id,src);
    	if(type.equals(PROVIDER_DATATREE)) return buildTree(id,src);

    	if(type.equals(PROVIDER_DATALIST_SECURED)) return buildListSecured(id,src);
    	if(type.equals(PROVIDER_DATASET_SECURED)) return buildSetSecured(id,src);
    	if(type.equals(PROVIDER_DATAMAP_SECURED)) return buildMapSecured(id,src);
    	if(type.equals(PROVIDER_DATATREE_SECURED)) return buildTreeSecured(id,src);

    	throw new Exception("Unknown data type: ["+type+"]");
	}
	
	
	
	
	
	
	
	
	private void registerSource(String id, Object src)
	{map_dataSource().put(id,src);}
	
	
	private void registerData(String id, Object data)
	{map_dataObject().put(id,data);}
	
	

	
	
	
	
	
	private List buildList(String id, Object src) throws Exception
	{
		if(src==null)					return new List1_n(id,new ArrayList());
    	if(src instanceof URL)			return new List1_n(id,list_U((URL)src));
    	if(src instanceof File)			return new List1_p(id,list_U((File)src));
    	if(src instanceof InputStream)	return new List1_n(id,list_U(text((InputStream)src)));
    	if(src instanceof String)		return new List1_n(id,list_U((String)src));
    	if(src instanceof List)			return new List1_n(id,(List)src);
    	if(src instanceof Provide)		return (List)((Provide)src).provide();
    	
    	throw new Exception("Invalid source for list: ["+src.getClass().getName()+"]");
	}
	
	
	
	
	private Set buildSet(String id, Object src) throws Exception
	{
		if(src==null)					return new Set1_n(id,new HashSet());
    	if(src instanceof URL)			return new Set1_n(id,set_U((URL)src));
    	if(src instanceof File)			return new Set1_p(id,set_U((File)src));
    	if(src instanceof InputStream)	return new Set1_n(id,set_U(text((InputStream)src)));
    	if(src instanceof String)		return new Set1_n(id,set_U((String)src));
    	if(src instanceof Set)			return new Set1_n(id,(Set)src);
    	if(src instanceof Provide)		return (Set)((Provide)src).provide();
    	
    	throw new Exception("Invalid source for set: ["+src.getClass().getName()+"]");
	}
	
	
	
	private Map buildMap(String id, Object src) throws Exception
	{
		if(src==null)					return new Map1_n(id,new Properties());
    	if(src instanceof URL)			return new Map1_n(id,prop((URL)src));
    	if(src instanceof File)			return new Map1_p(id,prop((File)src));
    	if(src instanceof InputStream)	return new Map1_n(id,prop((InputStream)src));
    	if(src instanceof Properties)	return new Map1_n(id,(Properties)src);
    	if(src instanceof Provide)		return (Map)((Provide)src).provide();
    	
    	throw new Exception("Invalid source for map: ["+src.getClass().getName()+"]");
	}
	
	
	
	private TreeModel buildTree(String id, Object src) throws Exception
	{
		if(src==null)					return new Tree1_n(id,"");
    	if(src instanceof URL)			return new Tree1_n(id,text_U((URL)src));
    	if(src instanceof File)			return new Tree1_p(id,text_U((File)src));
    	if(src instanceof InputStream)	return new Tree1_n(id,decodeU(text((InputStream)src)));
    	if(src instanceof String)		return new Tree1_n(id,decodeU((String)src));
    	if(src instanceof Provide)		return (TreeModel)((Provide)src).provide();
    	
    	throw new Exception("Invalid source for treeModel: ["+src.getClass().getName()+"]");
	}
	
	
	
	
	
	
	
	
	
	
	private Object buildListSecured(String id, Object src) throws Exception
	{
		if(src instanceof File)		return new List1_ps(id,list_US((File)src));
    	
    	throw new Exception("Invalid source for secured list: ["+src.getClass().getName()+"]");
	}
	
	private Object buildSetSecured(String id, Object src) throws Exception
	{
		if(src instanceof File)		return new Set1_ps(id,set_US((File)src));
    	
    	throw new Exception("Invalid source for secured set: ["+src.getClass().getName()+"]");
	}
	
	private Object buildMapSecured(String id, Object src) throws Exception
	{
		if(src instanceof File)		return new Map1_ps(id,prop_S((File)src));
    	
    	throw new Exception("Invalid source for secured map: ["+src.getClass().getName()+"]");
	}
	
	private Object buildTreeSecured(String id, Object src) throws Exception
	{
    	if(src instanceof File)		return new Tree1_ps(id,text_US((File)src));
    	
    	throw new Exception("Invalid source for secured treeModel: ["+src.getClass().getName()+"]");
	}
	
	
	
	
	
	
	
	
	
	
	
	
	private void persist(String id)
    {
    	try
    	{
    		Give g = dataPersistence();
    		if(g!=null) g.give(id);
    	}
    	catch(Exception e)
    	{err(this,"persist(String)",e);}
    }
    
	
	
	
	
	
	
	
	
	
	
	
	
	
	private class List1_n extends List1
    {    
        public List1_n(String id, List list)
        {super(id,list);}
        
        protected void modified(String eventID)
        {send(this,eventID);}

        protected void save(Object obj) throws Exception {}
    }
    
    private class List1_p extends List1_n
    {    
        public List1_p(String id, List list)
        {super(id,list);}
        
        protected void modified(String eventID)
        {
        	persist(id);
        	send(this,eventID);
        }
        protected synchronized void save(Object obj) throws Exception
        {writeList_U((File)obj,list);}
    }
    
    private class List1_ps extends List1_p
    {    
        public List1_ps(String id, List list)
        {super(id,list);}
        
        protected synchronized void save(Object obj) throws Exception
        {writeList_US((File)obj,list);}
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private class Set1_n extends Set1
    {
    	public Set1_n(String id, Set set)
        {super(id,set);}
    	
        protected void modified(String eventID)
        {send(this,eventID);}
        
        protected void save(Object obj) throws Exception {}
    }
    
    private class Set1_p extends Set1_n
    {
    	public Set1_p(String id, Set set)
        {super(id,set);}
    	
        protected void modified(String eventID)
        {
        	persist(id);
        	send(this,eventID);
        }
        protected synchronized void save(Object obj) throws Exception
        {writeSet_U((File) obj,set);}
    }
    
    private class Set1_ps extends Set1_p
    {
    	public Set1_ps(String id, Set set)
        {super(id,set);}
    	
        protected synchronized void save(Object obj) throws Exception
        {writeSet_US((File) obj,set);}
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    private class Map1_n extends Map1
    {
        public Map1_n(String id, Properties prop)
        {super(id,prop);}
        
        protected void modified(String eventID)
        {send(this,eventID);}
        
        protected void save(Object obj) throws Exception {}
    }
    
    private class Map1_p extends Map1_n
    {
        public Map1_p(String id, Properties prop)
        {super(id,prop);}
        
        protected void modified(String eventID)
        {
        	persist(id);
        	send(this,eventID);
        }
        protected synchronized void save(Object obj) throws Exception
        {writeProp((File)obj,prop);}
    }
    
    private class Map1_ps extends Map1_p
    {
        public Map1_ps(String id, Properties prop)
        {super(id,prop);}
        
        protected synchronized void save(Object obj) throws Exception
        {writeProp_S((File)obj,prop);}
    }
    
    
    
    
    
    
    
    
    
    
    
    public class Tree1_n extends Tree1
    {
        public Tree1_n(String id, String value) throws Exception
        {super(id,value);}
        
        protected void modified(String eventID)
        {send(eventID);}
        
        protected void save(Object obj) throws Exception {}
    }
    
    public class Tree1_p extends Tree1_n
    {
    	public Tree1_p(String id, String value) throws Exception
        {super(id,value);}
    	
        protected void modified(String eventID)
        {
        	persist(id);
        	send(eventID);
        }
        protected void save(Object obj) throws Exception
        {
        	PrintStream out = printStreamU((File)obj);
        	printTree(out);
        	out.close();
        }
    }
    
    public class Tree1_ps extends Tree1_p
    {
    	public Tree1_ps(String id, String value) throws Exception
        {super(id,value);}
    	
        protected void save(Object obj) throws Exception
        {
        	PrintStream out = printStreamUS((File)obj);
        	printTree(out);
        	out.close();
        }
    }
}
